DartOutput class final

Configuration for outputting generated Dart bindings.

Constructors

DartOutput({required Uri path})
const

Properties

hashCode → int
The hash code for this object.
no setterinherited
path → Uri
Path to write generated Dart bindings.
final
runtimeType → Type
A representation of the runtime type of the object.
no setterinherited

Methods

noSuchMethod(Invocation invocation) → dynamic
Invoked when a nonexistent method or property is accessed.
inherited
toString() → String
A string representation of this object.
inherited

Operators

operator ==(Object other) → bool
The equality operator.
inherited