DeclNode class abstract

Base class for declaration AST nodes.

Inheritance
Implementers

Constructors

DeclNode()
const

Properties

hashCode → int
The hash code for this object.
no setterinherited
name ↔ String
The generated Dart name for this AST node.
getter/setter pairinherited
originalName → String
Original C/C++/Objective-C name of this AST node.
no setterinherited
runtimeType → Type
A representation of the runtime type of the object.
no setterinherited
usr → String
USR identifier for this declaration.
no setter

Methods

accept(Visitor visitor) → void
inherited
noSuchMethod(Invocation invocation) → dynamic
Invoked when a nonexistent method or property is accessed.
inherited
toString() → String
A string representation of this object.
inherited

Operators

operator ==(Object other) → bool
The equality operator.
inherited