visitChildren method

  1. @override
void visitChildren(
  1. Visitor visitor
)

Visit this node's children. This is the method that actually understands the structure of the node. It should invoke Visitor.visit etc on all the node's children.

This method must be implemented if the node has children. The implementation should call super.visitChildren(visitor);.

Implementation

@override
void visitChildren(Visitor visitor) {
  super.visitChildren(visitor);
  visitor.visit(libraryImport);
}